from subprocess     import check_call
from subprocess     import CalledProcessError
from subprocess     import STDOUT
from subprocess     import check_output
from shutil         import copyfile
from shutil         import rmtree
from shutil         import chown
from os             import devnull
from os             import rename
from os             import path
from os             import remove
from os             import environ
from os             import listdir
from os             import system
from os             import chmod
from vpn_constants  import *

import re
import time
import syslog
import fcntl
import sqlite3

def setting_from_line(line):
  stripped_line = line.strip()
  # Skip empty lines
  if len(stripped_line) == 0:
    return None
  # Skip comments
  if '#' in stripped_line[0]:
    return None
  if '=' in line:
    (key, val) = line.split('=')
    return (key.strip(), val.strip())

####################################################################
# Class for making basic checks of certificates and keys.
class OpenSSLChecks:

  def cert_matches_key(cert_path, key_path):
    # Verify that the key and certificate match.
    cmd = '''var=$(openssl pkey -outform pem -pubout -in "'''+key_path+'''" | openssl sha256);
             var2=$(openssl x509 -outform pem -pubkey -in "'''+cert_path+'''" -noout | openssl sha256);
             echo "$var" | grep -q "$var2"'''
    return 0 == system(cmd)

  def is_certificate_file(cert_path):
    return 0 == system('openssl x509 -in '+cert_path+' -text -noout >/dev/null')

  def is_key_file(key_path):
    return 0 == system('openssl pkey -inform PEM -in '+key_path+' -text -noout >/dev/null')

  def is_encrypted_file(key_path):
    try:
      with open(key_path) as f:
        line = f.readline()
        while line:
          if line[0:10] == '-----BEGIN':
            line = f.readline()
            f.close()
            return line and line.find('ENCRYPTED') > -1
          else:
            line = f.readline()
        f.close()
    except:
      return False

####################################################################
# Class for management of user settings.
class SettingsFile:
  def __init__(self, path='settings.user', create=False):
    self.path = path
    self.create = create

  def write(self, name, value):
    new_lines = []
    replaced = False
    try:
      with open(self.path, 'r') as f:
        for line in f:
          s = setting_from_line(line)
          if s:
            (key, val) = s
            if key == name:
              new_lines.append(key + ' = ' + value + '\n')
              replaced = True
              continue
          new_lines.append(line)
    except FileNotFoundError:
      pass

    if not replaced:
      # We are adding a new setting, not changing a value of an existing one.
      new_lines.append(name + ' = ' + value + '\n')

    with open(self.path, 'w+') as f:
      for nline in new_lines:
        f.write(nline)

  def is_empty(self, name):
    if self.read(name):
      return False # Settings is either completely missing or has an empty value.
    return True

  def read(self, name):
    try:
      with open(self.path, 'r') as f:
        for line in f:
          s = setting_from_line(line)
          if s:
            (key, val) = s
            if key == name:
              return val
        return ""
    except Exception as e:
      if isinstance(e, FileNotFoundError) and self.create:
        return ""
      else:
        raise e

  def read_all(self):
    d = {}
    try:
      file_to_dict(self.path, d)
    except Exception as e:
      if not isinstance(e, FileNotFoundError) or not self.create:
        raise e
    return d

###############################################################################
##
##
class InstallProgress:
  def __init__(self, progress_file = '/tmp/wavpn-install.progress'):
    self.steps_done = 0
    self.progress_file = progress_file
    self.step_count = 0

  def set_steps(self, step_count):
    self.step_count = step_count

  def step(self, count = 1):
    self.steps_done += count
    if self.step_count != 0:
      self.write_progress(str(round(100*self.steps_done/self.step_count)))

  def progress_final_check(self):
    if (self.steps_done != self.step_count):
      warning_message = 'Steps done (' + str(self.steps_done) + ') differ from declared count of steps (' + str(self.step_count) + ').' 
      syslog.syslog(syslog.LOG_WARNING, warning_message)
      print('WARNING: ' + str(warning_message))

  def write_progress(self, status):
    with open(self.progress_file, 'w') as fifo:
      fifo.write(status)


# Repo constants
g_repo_sleep = 15 # Number of seconds to sleep between repository tries.

# Supported platforms
g_supported_platforms = {}
g_supported_platforms['ubuntu'] = ['16.04', '18.04', '20.04']
g_supported_platforms['debian'] = ['9', ]

g_os_codenames = {}
g_os_codenames['debian'] = {}
g_os_codenames['debian']['8']  = 'jessie'
g_os_codenames['debian']['9']  = 'stretch'
g_os_codenames['debian']['10'] = 'buster'

# Dirs and files
g_dir_debs               = './debs/'
g_dir_certificates       = './certificates/'
g_dir_vpnportal_dst      = '/opt/vpnportal/'
g_dir_cs_dst             = g_dir_vpnportal_dst + 'customer_server/'
g_dir_ovpn_cli_config    = g_dir_cs_dst + 'cli-config-dir/'
g_dir_cs_database_dst    = g_dir_cs_dst + 'database/'
g_file_cs_db             = g_dir_cs_database_dst + 'database.db'
g_file_cs_settings       = g_dir_cs_database_dst + 'settings.db'
g_file_user_settings     = 'settings.user'
g_file_version_suffix    = 'version.suffix'
g_file_web_settings      = 'settings.web'
g_dir_etc                = '/etc/'
g_dir_etc_apache2        = g_dir_etc + 'apache2/'
g_dir_apache2_ssl        = g_dir_etc_apache2 + 'ssl/'
g_dir_apache2_sites_en   = g_dir_etc_apache2 + 'sites-enabled/'
g_dir_apache2_sites_av   = g_dir_etc_apache2 + 'sites-available/'
g_dir_apache2_proxy      = g_dir_apache2_sites_av + 'vpnportal-proxy/'
g_dir_nagios             = g_dir_etc + 'nagios/'
g_file_cs_proxy_single   = g_dir_apache2_proxy + 'single.conf'
g_file_cs_proxy_multi    = g_dir_apache2_proxy + 'multi.conf'
g_file_apache2_def_ssl   = g_dir_apache2_sites_av + 'default-ssl.conf'
g_file_apache2_000_ssl   = g_dir_apache2_sites_en + '000-default_ssl.conf'
g_file_apache2_conf      = g_dir_apache2_sites_av + 'vpnportal-ssl.conf'
g_file_apache2_conf_tmp  = g_file_apache2_conf + '.template'
g_file_apache2_ssl_cert  = g_dir_apache2_ssl + 'web.crt'
g_file_apache2_ssl_key   = g_dir_apache2_ssl + 'web.pem'
g_file_proxy_single_conf = g_dir_apache2_proxy + "single.conf"
g_file_proxy_multi_conf  = g_dir_apache2_proxy + "multi.conf"
g_file_proxy_template    = g_dir_apache2_proxy + "proxy.conf.template"
g_file_nrpe_conf         = g_dir_nagios + 'nrpe.cfg'
g_file_nrpe_conf_tmp     = '/tmp/nrpe.cfg.tmp'
g_file_osrelease         = g_dir_etc + 'os-release'
g_dir_php                = g_dir_etc + 'php/'
g_dir_web                = '/var/www/vpnportal/'
g_file_web_db            = g_dir_web + 'www/db/database_prod.db'
g_file_envvars           = g_dir_etc_apache2 + 'envvars'
g_dir_certificates_dst   = g_dir_cs_dst + "/certificates/"
g_dir_rw_cli_certs       = g_dir_certificates_dst + "rw_clients/"
g_dir_router_cli_certs   = g_dir_certificates_dst + "ovpn_clients/"
g_dir_ca                 = g_dir_certificates_dst + "ca/"

# Definitions of service
g_service_cs             = 'vpnportal-cs.service'
g_service_ds             = 'vpnportal-ds.service'
g_service_rw             = 'roadwarrior.service'
g_timer_aptdaily         = 'apt-daily.timer'
g_service_ovpn           = 'vpnportal-cs-ovpn.service'
g_service_nrpe           = 'nagios-nrpe-server.service'

g_vpnportal_services     = (
                           g_timer_aptdaily,
                           g_service_ds,
                           g_service_cs,
                           g_service_rw
                           )
g_repositories_ubuntu    = ('ppa:ondrej/php', 'ppa:ondrej/apache2', )

# Version constants
g_version_cs_suffix      = 'VERSION_CS_SUFFIX'

g_progress = InstallProgress()

def banner(message):
  print("####################################################")
  print("### " + message)
  print("####################################################")

def print_error(message):
  print("ERROR: " + message)

def exec_command(command):
  FNULL = open(devnull, 'w')
  environment = environ
  environment["DEBIAN_FRONTEND"] = "noninteractive"
  check_call(command, stdout=FNULL, stderr=STDOUT, env = environment)

def check_tools(tools):
  g_progress.step()
  success = True
  for tool in tools:
    print('Checking tool ' + tool + ': ', end='')
    try:
      exec_command(['which', tool])
      print('OK')
    except:
      print('Missing')
      success = False
  return success

def get_os_release():
  d = {}
  file_to_dict(g_file_osrelease, d)
  version = d['VERSION_ID'].replace('"','')
  return d['ID'], version

def get_os_codename(os, version):
  if os in g_os_codenames:
    if version in g_os_codenames[os]:
      return g_os_codenames[os][version]
  return None

def check_os():
  g_progress.step()
  name, version = get_os_release()
  if not name in g_supported_platforms:
    print("Not supported platform")
    return False
  if not version in g_supported_platforms[name]:
    print("Not supported version.")
    return False
  return True

def systemd_reload():
  exec_command(["systemctl","daemon-reload"])

def is_file_locked(in_file):
  with open(in_file, 'w') as handle:
    try:
      fcntl.lockf(handle, fcntl.LOCK_EX | fcntl.LOCK_NB)
      return False
    except IOError:
      return True

def wait_4_file_unlock(in_file, timeout = 1):
  while is_file_locked(in_file):
    time.sleep(timeout)

def wait_4_apt_dies():
  g_progress.step()
  wait_4_file_unlock('/var/lib/dpkg/lock')
  wait_4_file_unlock('/var/lib/dpkg/lock-frontend')
  wait_4_file_unlock('/var/cache/apt/archives/lock')

def manage_package(package, action):
  exec_command(["apt-get", action, "-y", "-q", package])

def install_package(package):
  g_progress.step()
  manage_package(package, "install")

def install_packages(caption, packages):
  print(caption)
  for package in packages:
    print("\tInstalling " + package)
    install_package(package)

def remove_package(package):
  manage_package(package, "remove")

def remove_packages(caption, packages):
  print(caption)
  for package in packages:
    print("\tRemoving " + package)
    remove_package(package)

def purge_package(package):
  try:
    manage_package(package, "purge")
  except CalledProcessError as exception:
    # Return code 100 means that package was not installed so is not removed.
    # Fu*k this error.
    if exception.returncode != 100:
      raise exception
  finally:
    g_progress.step()

def purge_packages(caption, packages):
  print(caption)
  for package in packages:
    print("\tPurging " + package)
    purge_package(package)

def manage_deb_package(package, action):
  FNULL = open(devnull, 'w')
  exec_command(["dpkg", action,  package])

def install_deb_package(package):
  g_progress.step()
  manage_deb_package(package, "-i")

def install_deb_packages(caption, packages):
  print(caption)
  for name, version in packages:
    package = g_dir_debs + str(name) + str(version) + '.deb'
    print("\tInstalling " + package)
    install_deb_package(package)

def remove_deb_package(package):
  g_progress.step()
  manage_deb_package(package, "-P")

def remove_deb_packages(caption, packages):
  print(caption)
  for name, version in reversed(packages):
    print("\tRemoving " + name)
    remove_deb_package(name)

def untar_file(src_file, dest_dir):
  exec_command(['tar', '-xf', src_file, '-C', dest_dir])

# This function performs change of owner and group to root and vpnportal for
# all files and directories in 'dest_dir'.
def chown_vpnportal(dest_dir):
  exec_command(['chown', '-R', 'root:vpnportal', dest_dir])

# CONFIGURATION

# This function creates configuration file (from template file) 
# for apache2.
def create_apache2_config():
  c_apache2_variables_start = 'VARIABLES:START'
  with open(g_file_apache2_conf_tmp, "r") as conf_tmp_r:
    with open(g_file_apache2_conf, "w") as conf_w:
      for line in conf_tmp_r:
        conf_w.write(line)
        if c_apache2_variables_start in line:
          s = SettingsFile(g_file_user_settings)
          conf_w.write('  define ' + SETTINGS_CS_DOMAIN_NAME + ' ' + s.read(SETTINGS_CS_DOMAIN_NAME) + '\n')
  chown(g_file_apache2_conf, group='vpnportal')
  chmod(g_file_apache2_conf, 0o664)

def configure_php(version, max_size = '512M'):
  g_progress.step()
  configure_apache2_php(version, max_size)

def configure_apache2_php(version, max_size):
  c_phpini_file         = g_dir_php + '/' + str(version) + '/apache2/php.ini'
  c_phpini_tmp_file     = '/tmp/php.ini.tmp'
  c_post_max_size       = 'post_max_size'
  c_upload_max_filesize = 'upload_max_filesize'

  copyfile(c_phpini_file, c_phpini_tmp_file)
  with open(c_phpini_tmp_file, "r") as phpini_tmp_r:
    with open(c_phpini_file, "w") as phpini_w:
      for line in phpini_tmp_r:
        if c_post_max_size in line:
          phpini_w.write(str(c_post_max_size) + ' = ' + str(max_size) + '\n')
        elif c_upload_max_filesize in line:
          phpini_w.write(str(c_upload_max_filesize) + ' = ' + str(max_size) + '\n')
        else:
          phpini_w.write(line)

# This function generates RSA key.
def generate_ssl_key(file_out_key, key_len = 4096):
  exec_command(['openssl', 'genrsa', '-out', file_out_key, str(key_len)])

# This function generates certificate.
def generate_ssl_cert(file_key_sign,
                     file_out_cert,
         common_name,
         sha_bits          = 512,
         country           = None,
         state             = None,
         locality          = None,
         organization      = None,
         organization_unit = None,
         days              = '9999'):

  if not common_name:
    raise Exception('Missing Common Name for certificate generation.')

  subject = (('/C='  + country) if country else '') + \
            (('/ST=' + state) if state else '') + \
      (('/L='  + locality) if locality else '') + \
      (('/O='  + organization) if organization else '') + \
      (('/OU=' + organization_unit) if organization_unit else '') + \
       ('/CN=' + common_name)

  exec_command(['openssl', 'req', '-new', '-key', file_key_sign,
                '-out', '/tmp/private.csr',
          '-subj', subject
               ])

  exec_command(['openssl', 'x509',
                '-in',  '/tmp/private.csr',
                '-out', file_out_cert,
                '-req',
                '-signkey', file_key_sign,
                '-days', days,
                '-sha' + str(sha_bits)
               ])

  # Remove temporary CSR file.
  exec_command(['rm', '-f', '/tmp/private.csr'])

# This function generates SSL key and certificate for apache2.
def configure_apache2_ssl():
  # Create directory for SSL certificates.
  exec_command(['mkdir', '-p', g_dir_apache2_ssl])
  # Enable SSL mode.
  exec_command(['a2enmod', 'ssl'])
  # Enable headers mode.
  exec_command(['a2enmod', 'headers'])
  
def configure_apache2():
  g_progress.step()
  create_apache2_config()
  configure_apache2_ssl()
  # Create empty proxy files
  open(g_file_cs_proxy_single, 'a').close()
  open(g_file_cs_proxy_multi, 'a').close()
  chown(g_file_cs_proxy_single, group='vpnportal', user='www-data')
  chown(g_file_cs_proxy_multi,  group='vpnportal', user='www-data')
  chmod(g_file_cs_proxy_single, 0o664)
  chmod(g_file_cs_proxy_multi,  0o664)

def create_nrpe_config():
  c_allowed_hosts = 'allowed_hosts'
  copyfile(g_file_nrpe_conf, g_file_nrpe_conf_tmp)
  with open(g_file_nrpe_conf_tmp, "r") as conf_tmp_r:
    with open(g_file_nrpe_conf, "w") as conf_w:
      for line in conf_tmp_r:
        if c_allowed_hosts in line:
          s = SettingsFile(g_file_user_settings)
          nagios_ip = s.read(SETTINGS_NAGIOS_SERVER_IP)
          if nagios_ip:
            allowed_hosts_str = line.strip() + ', ' + nagios_ip + '\n'
            conf_w.write(allowed_hosts_str)
            continue
        conf_w.write(line)


def configure_nagios():
  g_progress.step()
  banner("Configuring Nagios")
  create_nrpe_config()
  restart_nrpe()
  rmtree(g_file_nrpe_conf_tmp, ignore_errors=True)

def add_group(group):
  exec_command(['groupadd', '-f', group]);

def add_user(user, group):
  exec_command(['usermod', '-a', '-G', group, user]);

def configure_groups_and_users():
  g_progress.step()
  add_group('vpnportal')
  add_group('nagios')
  add_group('nagcmd')
  add_user('www-data', 'vpnportal')
  add_user('www-data', 'adm')
  add_user('www-data', 'nagios')
  add_user('www-data', 'nagcmd')

def manage_repository(repository, action = ''):
  if action == '':
    exec_command(["add-apt-repository", "-y", repository])
  else:
    exec_command(["add-apt-repository", "-y", action, repository])

def manage_repositories_ubuntu(repositories, action = '', prefix = ''):
  for repository in repositories:
    print(str(prefix) + 'Adding ', end = '') if action == '' else print(str(prefix) + 'Removing ', end = '')
    print('repository: ' + str(repository))
    while True:
      try:
        manage_repository(repository, action)
        break
      except CalledProcessError as exception:
        print(str(prefix) + 'Failed. Next attempt in ' + str(g_repo_sleep) + ' s.')
        time.sleep(g_repo_sleep)

  print(str(prefix) + 'Running update')
  exec_command(["apt-get", "update"])

def add_repositories_ubuntu(repositories, prefix = ''):
  manage_repositories_ubuntu(repositories, prefix = prefix)

def remove_repositories_ubuntu(repositories, prefix = ''):
  manage_repositories_ubuntu(repositories, '-r', prefix)

def add_repositories_debian():
  exec_command(['bash', 'add-debian-repos.sh'])

def add_repositories():
  g_progress.step()
  name, version = get_os_release()
  if name == 'ubuntu':
    add_repositories_ubuntu(g_repositories_ubuntu, '\t')
  elif name == 'debian':
    add_repositories_debian()
  else:
    print('Not able to add repositories for ' + str(name))

def remove_repositories():
  name, version = get_os_release()
  if name == 'ubuntu':
    remove_repositories_ubuntu(g_repositories_ubuntu, '\t')

# This method configures proc file system settings:
# * IPv4 IP forward: 1
def proc_configure():
  g_progress.step()
  exec_command(["sysctl", "-w", "net.ipv4.ip_forward=1"])

  found = False
  pattern = re.compile('^\s*net.ipv4.ip_forward')

  for i, line in enumerate(open('/etc/sysctl.conf')):
    for match in re.finditer(pattern, line):
      found = True
  if not found:
    with open('/etc/sysctl.conf', 'a') as f:
      f.write('net.ipv4.ip_forward=1\n')

# This method checks if line is present in file.
def line_in_file(infile, line):
  with open(infile,'r') as f:
    for l in f:
      if line in l:
        return True

  return False

# This method adds 'user' to sudoers file.
def sudoers_config():
  g_progress.step()
  FNULL = open(devnull, 'w')
  sudoers = "%vpnportal ALL = NOPASSWD:/usr/bin/systemctl,/usr/bin/openssl,/usr/bin/rm"
  cmd = "echo '" + sudoers + "' | sudo EDITOR='tee -a' visudo"
  found = False

  if not line_in_file('/etc/sudoers', sudoers):
    check_call(cmd, shell=True, stdout=FNULL)

def service_start(service):
  exec_command(["systemctl", "start", service])

def service_restart(service):
  exec_command(["systemctl", "restart", service])

def service_stop(service):
  exec_command(["systemctl", "stop", service])

def services_start(services):
  g_progress.step()
  for service in services:
    service_start(service)

def service_enable(service):
  exec_command(["systemctl", "enable", service])

def services_enable(services):
  g_progress.step()
  for service in services:
    service_enable(service)

def services_stop(services):
  g_progress.step()
  for service in reversed(services):
    service_stop(service)

def restart_apache2():
  service_restart('apache2')

def restart_nrpe():
  service_restart('nagios-nrpe-server')

def file_to_dict(in_file, dictionary):

  if not path.isfile(in_file):
    return False

  with open(in_file, 'r') as in_f:
    for line in in_f:
      stripped_line = line.strip()
      # Skip empty lines
      if len(stripped_line) == 0:
        continue
      # Skip comments
      if '#' in stripped_line[0]:
        continue
      if '=' in line:
        (key, val) = line.split('=')
        dictionary[key.strip()] = val.strip()
  return True

def version_suffix_update(value):
  conn = sqlite3.connect(g_file_cs_db)
  cursor = conn.cursor()

  sql_cmd = 'UPDATE version_table SET suffix = "' + value + '" WHERE item = "CS"'
  cursor.execute(sql_cmd)
  conn.commit()
  conn.close()

def settings_update(key, value):
  conn = sqlite3.connect(g_file_cs_settings)
  cursor = conn.cursor()

  sql_cmd = 'UPDATE settings_table SET value = "' + value + '" WHERE name = "' + key + '"'
  cursor.execute(sql_cmd)
  conn.commit()
  conn.close()

def apply_db_settings():
  g_progress.step()
  d = {}
  if file_to_dict(g_file_version_suffix, d):
    banner("Version suffix found. Applying...")
    if g_version_cs_suffix in d:
      cs_version_suffix = d[g_version_cs_suffix]
      print("\tUpdating CS version suffix: " + cs_version_suffix)
      version_suffix_update(cs_version_suffix)

def apply_user_settings():
  g_progress.step()
  d = {}
  if file_to_dict(g_file_user_settings, d):
    banner("User settings found. Applying...")
    if SETTINGS_DS_IP in d:
      print("\tSetting DS IP: " + d[SETTINGS_DS_IP])
      settings_update(SETTINGS_DS_IP, d[SETTINGS_DS_IP])
    if SETTINGS_CS_REGISTER_IP in d:
      print("\tSetting CS registration IP: " + d[SETTINGS_CS_REGISTER_IP])
      settings_update(SETTINGS_CS_REGISTER_IP, d[SETTINGS_CS_REGISTER_IP])
    if SETTINGS_SYSLOG_LOGLEVEL in d:
      print("\tSetting CS loglevel: " + d[SETTINGS_SYSLOG_LOGLEVEL])
      settings_update(SETTINGS_SYSLOG_LOGLEVEL, d[SETTINGS_SYSLOG_LOGLEVEL])
    if SETTINGS_OPENVPN_SERVER1_NETWORK in d:
      print("\tSetting OVPN network ip: " + d[SETTINGS_OPENVPN_SERVER1_NETWORK])
      settings_update(SETTINGS_OPENVPN_SERVER1_NETWORK, d[SETTINGS_OPENVPN_SERVER1_NETWORK])
    if SETTINGS_OPENVPN_SERVER2_MASK in d:
      print("\tSetting OVPN network mask: " + d[SETTINGS_OPENVPN_SERVER2_MASK])
      settings_update(SETTINGS_OPENVPN_SERVER2_MASK, d[SETTINGS_OPENVPN_SERVER2_MASK])
    # Domain name is mandatory. If it is missing, then we print warning
    # and configure a default value. 
    print("\tSetting CS domain name: " + d[SETTINGS_CS_DOMAIN_NAME])
    if SETTINGS_CS_DOMAIN_NAME in d:      
      settings_update(SETTINGS_CS_DOMAIN_NAME, d[SETTINGS_CS_DOMAIN_NAME])
    else:
      print("\tWarning: Domain name is missing. Using 'localhost' as default value.")
      settings_update(SETTINGS_CS_DOMAIN_NAME, "localhost")
      
# Generation of SSL certificates
'''
This function is used to execute PHP code.
'''
def exec_php(php_code):
  cmd = ['php', '-r', php_code]
  exec_command(cmd)

'''
This functions checks if CA settings are present in user settings file.
'''
def check_ca_user_settings():
  s = SettingsFile(g_file_user_settings)
  if s.is_empty(SETTINGS_CS_DN_CN):
    return False
  return True

'''
This function returns True if there are certificates to import in
installation package.
'''
def check_ca_importable_certificates(dir_certs = './certificates'):
  # Check if certificate directory exists.
  if not path.isdir(dir_certs):
    return False
  # Check if CA keypair exists.
  if not path.isfile(dir_certs + '/' + 'ca.pem'):
    return False
  if not path.isfile(dir_certs + '/' + 'ca.crt'):
    return False
  return True

'''
This function generates full set of certificates (including CA)
based on user settings.
'''
def generate_certificates():
  s = SettingsFile(g_file_user_settings)
  php_code = '$phpclientapi = new CSPhpClientApi(); ' \
             '$phpclientapi->create_new_ca(' \
                 '"' + s.read(SETTINGS_CS_DN_CN)           + '", ' \
                 '"' + s.read(SETTINGS_CS_DN_COUNTRY)      + '", ' \
                 '"' + s.read(SETTINGS_CS_DN_LOCALITY)     + '", ' \
                 '"' + s.read(SETTINGS_CS_DN_ORGANIZATION) + '", ' \
                 '"' + s.read(SETTINGS_CS_DN_EMAIL)        + '", ' \
                 '9999' + \
              ');'
  try:
    exec_php(php_code)
  except CalledProcessError as exception:
    print("Error during generating certificates (see syslog for more details).")
    print(php_code)

'''
This function imports certificates from installation package.
'''
def import_certificates(dir_import = './certificates'):
  php_code = '$phpclientapi = new CSPhpClientApi(); ' \
             '$phpclientapi->import_certificates("' + dir_import + '");'
  try:
    exec_php(php_code)
  except CalledProcessError as exception:
    print("Error during importing certificates (see syslog for more details).")

def install_certificates():
  g_progress.step()
  if check_ca_importable_certificates():
    banner("Importing certificates")
    import_certificates()
  else:
    banner("Generating certificates")
    if check_ca_user_settings():
      generate_certificates()
    else:
      print("Error: Value of '"+SETTINGS_CS_DN_CN+"' is missing in '"+g_file_user_settings+\
            "'. Aborting creation of certificates...")
  restart_apache2()

'''
This function sets web admin password.
It reads it from settings.web file, hashes it and updates database.
'''
def webadmin_password_set():
  g_progress.step()
  try:
    s = SettingsFile(g_file_web_settings)
    password = s.read('WEB_ADMIN_PASSWORD').encode('utf-8')
    if password:
      php_code = '$phpclientapi = new CSPhpClientApi(); ' \
                 '$phpclientapi->set_web_passwd("admin", '+ str(password) +');'
      exec_php(php_code)
      print('Web admin password was set.')
  except FileNotFoundError:
    pass # Web settings file is not present    
  except CalledProcessError as exception:
    print("Error during update of admin's password.")

